<!DOCTYPE kpartgui>
<kpartgui name="kgpg" version="33">
<MenuBar>
  <Menu name="file">
    <Action name="file_new_window"/>
    <Action name="file_encrypt"/>
    <Action name="file_decrypt"/>
    <Separator/>
    <Action name="key_manage"/>
  </Menu>

    <Menu name="view"><text>&amp;View</text>
    <Action name="charsets"/>
  </Menu>
  
  <Menu name="signature"><text>Si&amp;gnature</text>
    <Action name="sign_generate"/>
    <Action name="sign_verify"/>
    <Action name="sign_check"/>
  </Menu>
  
  	<Menu name="settings"><text>&amp;Settings</text>
	<Action name="kgpg_config" />
	</Menu>
	
</MenuBar>
</kpartgui>
