<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="khexedit2part" version="1">
<MenuBar>
  <Menu name="edit">
    <Action name="edit_copy" group="edit_paste_merge" />
    <Action name="edit_select_all" group="edit_select_merge" />
    <Action name="edit_deselect" group="edit_select_merge" />
  </Menu>
  <Menu name="view">
    <Separator/>
    <Action name="view_zoom_in" />
    <Action name="view_zoom_out" />
    <Separator/>
    <Action name="view_showunprintable" />
    <Action name="view_lineoffset" />
    <Separator/>
    <Action name="view_valuecoding" />
    <Action name="view_charencoding" />
  </Menu>
  <Menu name="settings">
    <Separator/>
    <Action name="resizestyle" group="show_merge" />
    <Action name="togglecolumns" group="show_merge" />
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Action name="edit_copy" />
  <Action name="view_zoom_in" />
  <Action name="view_zoom_out" />
</ToolBar>

<!-- Editor popup menu //-->
<Menu name="editor_popup">
  <Action name="view_hexcoding" />
  <Action name="view_deccoding" />
  <Action name="view_octcoding" />
  <Action name="view_bincoding" />
  <Separator/>
  <Action name="resize_lock" />
</Menu>
</kpartgui>
