<!DOCTYPE kpartgui>
<kpartgui name="kcalc" version="16">
<MenuBar>
  <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
    <Action name="show_trig"/>
    <Action name="show_stat"/>
    <Action name="show_logic"/>
    <Action name="show_explog"/>
    <Action name="show_constants"/>
    <Separator/>
    <Action name="show_all"/>
    <Action name="hide_all"/>
    <Separator/>
    <Action name="options_configure_keybinding"/>
    <Action name="options_configure"/>
  </Menu>
</MenuBar>
</kpartgui>
