<!DOCTYPE kpartgui>
<kpartgui name="ark" version="9">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="new_window"/>
    <Action name="reload_arch"/>
    <Merge/>
   </Menu>
  <Menu name="settings">
    <DefineGroup name="settings_show" append="show_merge"/>
    <DefineGroup name="settings_configure" append="configure_merge"/>
  </Menu>
  <Merge/>
</MenuBar>
<Menu name="archive_popup">
 <Action name="file_new"/>
 <Action name="file_open"/>
 <Action name="file_close"/>
 <Separator lineSeparator="true"/>
 <Merge/>
</Menu>
</kpartgui>
